package com.sxtvs.open.api.review.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.ImmutableMap;
import com.sxtvs.open.api.h5template.service.impl.OSSService;
import com.sxtvs.open.api.review.dto.PageReqDTO;
import com.sxtvs.open.api.review.dto.VideoReviewResp;
import com.sxtvs.open.api.review.entity.ReviewImgRecord;
import com.sxtvs.open.api.review.entity.ReviewVideosRecord;
import com.sxtvs.open.api.review.mapper.ReviewVideosRecordMapper;
import com.sxtvs.open.api.review.service.IReviewVideosRecordService;
import com.sxtvs.open.core.advice.BizException;
import com.sxtvs.open.core.auth.AESUtil;
import com.sxtvs.open.core.auth.HttpContextUtil;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.util.TextUtils;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Optional;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2024-02-05
 */
@Service
@Slf4j
public class ReviewVideosRecordServiceImpl extends ServiceImpl<ReviewVideosRecordMapper, ReviewVideosRecord> implements IReviewVideosRecordService {

    @Resource
    private VcrService vcrService;

    @Resource
    private OSSService  ossService;

    @Resource
    private SSEService sseService;

    public ReviewVideosRecord store(ReviewVideosRecord reviewVideosRecord) {
        Optional<String> token = HttpContextUtil.getToken();

        String uid = AESUtil.decryptStr(token.get());
        reviewVideosRecord.setUid(Long.valueOf(uid));
        reviewVideosRecord.setCreateTime(LocalDateTime.now());
        ResponseEntity<VideoReviewResp> resp = vcrService.reviewVideo(JSON.toJSONString(ImmutableMap.of("source", reviewVideosRecord.getUrl())));
        VideoReviewResp body = resp.getBody();

        if (null == body){
            log.error("视频审核失败 {}", resp.getStatusCode());
            this.save(reviewVideosRecord);
            throw new BizException("视频审核失败,系统正在尝试重新提交 SHARES稍后");
        }
        body.fill();
        if(TextUtils.isEmpty(reviewVideosRecord.getTitle())){
            String[] extension = reviewVideosRecord.getUrl().split("/");
            reviewVideosRecord.setTitle(extension[extension.length - 1]);
        }
        reviewVideosRecord.setTaskId(body.getTaskId());
        reviewVideosRecord.setStatus(body.getStatus());
        reviewVideosRecord.setResult(JSON.toJSONString(body));
        reviewVideosRecord.setReviewResult(body);
        reviewVideosRecord.setCover(reviewVideosRecord.getUrl() + "?x-oss-process=video/snapshot,t_1000,m_fast");
        this.save(reviewVideosRecord);
        return reviewVideosRecord;
    }

    private void refreshResult(ReviewVideosRecord reviewVideosRecord) {
        ResponseEntity<VideoReviewResp> resp = vcrService.getVideoResult(reviewVideosRecord.getTaskId());
        VideoReviewResp body = resp.getBody();
        if (null == body){
            log.error("视频审核失败 {}", resp.getStatusCode());
            throw new BizException("视频审核失败,系统正在尝试重新提交 请稍后");
        }

        body.fill();
        reviewVideosRecord.setStatus(body.getStatus());

        if ("SUCCESS".equals(body.getStatus())){
            // 图片本地化
            body.getResults().forEach(result -> result.getItems().forEach(item -> {
                VideoReviewResp.ResultsDTO.ItemsDTO.EvidenceDTO evidence = item.getEvidence();
                if (evidence != null && !TextUtils.isEmpty(evidence.getThumbnail())){
                    evidence.setThumbnail(ossService.vcr2local(evidence.getThumbnail(), reviewVideosRecord.getTaskId()));
                }
            }));
            if(sseService.sendMsg(reviewVideosRecord.getUid(),
                    ImmutableMap.of("type", "VIDEO", "msg",
                            "视频" + reviewVideosRecord.getTitle() + "审核完成")
            )){
                reviewVideosRecord.setPushStatus(1);
            }
        }

        reviewVideosRecord.setResult(JSON.toJSONString(body));

        updateById(reviewVideosRecord);

    }

    public void push(){
        lambdaQuery().eq(ReviewVideosRecord::getStatus, "SUCCESS").eq(ReviewVideosRecord::getPushStatus, 0).list().forEach(x -> {
            if(sseService.sendMsg(x.getUid(),
                    ImmutableMap.of("type", "VIDEO", "msg", "视频" + x.getTitle() + "审核完成")
            )){
                x.setPushStatus(1);
                updateById(x);
            }
        });
    }

    public void refreshBatchResult(){
        log.info("开始刷新视频审核结果");
        lambdaQuery().in(ReviewVideosRecord::getStatus, Arrays.asList("PROVISIONING", "PREPROCESSING", "PROCESSING")).list().forEach(this::refreshResult);
    }

    public Page<ReviewVideosRecord> list(PageReqDTO pageReqDTO){
        Page<ReviewVideosRecord> page = new Page<>(pageReqDTO.getPage(), pageReqDTO.getPageSize());
        Optional<String> token = HttpContextUtil.getToken();
        String uid = AESUtil.decryptStr(token.get());
        LambdaQueryWrapper<ReviewVideosRecord> wrapper = new LambdaQueryWrapper<>();
        return baseMapper.selectPage(page, wrapper.eq(ReviewVideosRecord::getUid, Long.valueOf(uid))
                .like(!TextUtils.isEmpty(pageReqDTO.getTitle()), ReviewVideosRecord::getTitle, pageReqDTO.getTitle()).orderByDesc(ReviewVideosRecord::getId));
    }

    public void delete(Long id) {
        Optional<String> token = HttpContextUtil.getToken();
        String uid = AESUtil.decryptStr(token.get());
        ReviewVideosRecord reviewVideosRecord = this.getById(id);
        if (!reviewVideosRecord.getUid().equals(Long.valueOf(uid))){
            throw new BizException("无权删除");
        }
        this.removeById(id);
    }
}
